# 
rm(list = ls())
setwd("C:/Users/Utente/OneDrive - University of Pisa/Esercitazioni UniPi/Lezioni JM 2018/Lezioni II semestre JM SAE/6 May 2019")
# sample 
sample<-read.csv("silc_austria.csv", header=T, sep=",")
# population mean at district level
pop<-read.csv("pop_m.csv", header=T, sep=",")

str(sample)

str(pop)

# The target variable in the example is the equivalized household income (eqIncome), 
# which is defined as the total household disposable income divided by the equivalized 
# household size determined by the modified scale of the OECD

# Target variable
# eqIncome The equivalized household income. 

# Domain identifiers
# state Austrian states. 
# district Austrian districts ("small areas")

# https://en.wikipedia.org/wiki/NUTS_statistical_regions_of_Austria

# Explanatory variables
# eqsize The equivalized household size 
# gender The person's gender (levels: female and male). 
# cash Employee cash or near cash income. 
# self_empl Cash benefits or losses from self-employment (net). 
# unempl_ben Unemployment benefits (net). 
# age_ben Old-age benefits (net). 
# surv_ben Survivor's benefits (net). 
# sick_ben Sickness benefits (net). 
# dis_ben Disability benefits (net). 
# rent Income from rental of a property or land (net). 
# fam_allow Family/children related allowances (net). 
# house_allow Housing allowances (net). 
# cap_inv Interest, dividends, profit from capital investments in 
# unincorporated business (net).
# tax_adj Repayments/receipts for tax adjustment (net). 

# Design variable
# weight Sampling weight. 

# order datasets
sample<-sample[order(sample [,"district"]),]
pop<-pop[order(pop [,"Group.1"]),]


# sample in each district
View(table(sample$district))


library(sae)
# calculate the direct estimates of eqIncome for districts
HT <- direct(eqIncome, dom=district,
             sweight=weight,
             domsize=aggregate(sample$weight, list(sample$district), sum),
             data=sample)
summary(HT$CV)

# categorize areas on the basis of CV
library(car)
cat_CV<-recode(HT$CV, "0:16.5= '< 16.5%'; 16.51:33.3='between 16.6% and 33.3% '; else='>33.3%'")
table(cat_CV)



# map the results
# load libraries useful for graph
library(maptools)
library(RColorBrewer)
library(classInt)
library(sp)
library(lattice)

# In order to plot a map we have to load the shape file corresponding to the Country of interest
# Shapefiles are so-called vector databases, describing the location of points 
# (e.g., collecting locations), polylines (e.g., roads)  andareas (or polygons, e.g., soil types,  
# countries). A shapefile actually consists of three separate files with the same name 
# but with different extensions (SHP, SHX and DBF), but they are treated as one file. 

# shape file are usually available online. For example
# you can use this website to download shape files
# http://www.diva-gis.org/datadown

# For Italian data you can download shape files from the ISTAT website

# use function readShapePoly() from package maptools
# The readShapePoly() reads data from a polygon shapefile into a SpatialPolygonsDataFrame object.
# inside the folder AUT_adm I have a series of file
# the file useful for me in this case is AUT_adm1.shp
Austria=readShapePoly("shp_aut\\shape_austria_dis.shp")
# You can obtain a warning meassage because the use of this function is 
# deprecated and it is not being maintained, but it works!
plot(Austria)
# merge results and shape file
AUT<- merge(Austria, HT, by.x="PB", by.y="Domain")

# spplot(AUT, "Direct", main = "Direct Estimate NUTS 3")
# number of classes
nclr <- 4
class <- classIntervals(AUT$Direct, nclr, style="quantile")
#breakpoints 
categ = class[["brks"]]

# RColorBrewer package for choose colors
# you can see a list of all the color pallets that come with RColorBrewer 
display.brewer.all()

plotclr <- brewer.pal(nclr,"YlOrRd")

# plotclr2 <- c("gray72","gray40", "gray17", "gray7")

# cut divides the range of x into intervals and codes the values in x according 
# to which interval they fall.

# right	: logical, indicating if the intervals should be closed on the right 
# (and open on the left) or vice versa.

# include.lowest: logical, indicating if an 'x[i]' equal to the lowest (or highest, 
# for right = FALSE) 'breaks' value should be included.

AUT$cutDIR <- cut(AUT$Direct, breaks = categ, 
                  right = F, include.lowest=T)  

# plot results with spplot
spplot(AUT, "cutDIR", col.regions=plotclr, 
       main = "Direct Estimates (NUTS 3)",
       colorkey = list(space="bottom",height = 0.9, 
                       labels = list(at = seq(0.5, length(categ) -0.5), 
                                     labels = round(categ,2))))


# unit level model
# linear model to check the significance of covariate
lm1 <- lm(eqIncome~gender + eqsize + cash + self_empl + unempl_ben +
            age_ben + surv_ben + sick_ben + dis_ben + rent + fam_allow + 
            house_allow + cap_inv + tax_adj, data=sample)
summary(lm1)

# look for significant covariates with a stepwise method
slm1 <- step(lm1,direction="both",trace=0)
summary(slm1)

# gender, eqsize, cash, self_empl, unempl_ben, age_ben, surv_ben,
# sick_ben, dis_ben, rent, house_allow, cap_inv, tax_adj

# usually in SAE we have a few additional covariates

# for simplicity (the goal of the lesson is to teach you how to perform
# SAE in R) we select just the first 6 covariates:
# gender: The person's gender (levels: female and male)
# eqsize: The equivalized household size
# cash: Employee cash or near cash income. 
# self_empl: Cash benefits or losses from self-employment (net). 
# unempl_ben: Unemployment benefits (net). 
# age_ben: Old-age benefits (net). 
lm2 <- lm(eqIncome~gender+ eqsize+ cash+ self_empl+ unempl_ben+age_ben, data=sample)
summary(lm2)

# now we compute the EBLUP at unit level
# read the vignettes of sae package!!
# https://cran.r-project.org/web/packages/sae/vignettes/sae_basicdirect.pdf
# https://cran.r-project.org/web/packages/sae/vignettes/sae_methodology.pdf
# and the article on The R Journal
# https://journal.r-project.org/archive/2015/RJ-2015-007/RJ-2015-007.pdf
# we need to compute the population size in each area
help(eblupBHF)
# call of the function
# eblupBHF(formula, dom, selectdom, meanxpop, popnsize, method = "REML", data)

# to compute the MSE we have to run the function pbmseBHF()
help(pbmseBHF)
# pbmseBHF(formula, dom, selectdom, meanxpop, popnsize, B = 200, method = "REML", data)

# The fixed part of the model needs to be specified through the argument formula and the variable
# (vector or factor) identifying the domains must be specified in the argument dom. 

# The functions allow selection of a subset of domains where we want to estimate by specifying the vector of
# selected domains in selectdom, which by default includes the list of all unique domains in dom. 
#
# population means of the auxiliary variables for the domains (meanxpop) and the population sizes of
# the domains (popnsize) are required arguments. REML (default) or ML fitting methods can be specified
# in argument method.

# population size
popN<-aggregate(sample$weight, list(sample$district), sum)

# EBLUP
unit.eblup1<-pbmseBHF(eqIncome~gender+ eqsize+ cash+ self_empl+ unempl_ben+age_ben,
                      dom=district, meanxpop=pop,
                      popnsize=popN, B=5, data=sample) 
# you should choose a larger value for B (bootstrap replications) (such as 500)

# for areas with zero sample size we can compute the synthethic estimator
# y.hay= Xb
# we have 24 areas with zero sample size
setd<-setdiff(pop$Group.1,sample$district)
setd
length(setd)

# select popolation mean for out of sample areas
X.m_tmp <- subset(pop, Group.1 %in% setd) 
X.m_o<-X.m_tmp[,-1]

# compute synthethic estimator
beta.hat<-unit.eblup1$est$fit$summary$coefficients[,1]
betaest<-matrix(beta.hat,nrow=7,ncol=1) #nrow= number of covariates + Intercept
meanxpop <- cbind(1,X.m_o)
eblup_out<- as.matrix(meanxpop)%*%betaest    
RESULTS_out<-data.frame(Domain=X.m_tmp$Group.1,
                        Eblup=eblup_out)

# variance can be obtain with smoothing method 
# (see Wolter, Introduction to Variance Estimation)

# calculate the CV %
CV.eblup1<-100*sqrt(unit.eblup1$mse$mse)/abs(unit.eblup1$est$eblup$eblup)
CV.eblup1_t<- recode(CV.eblup1, "0:16.5= '< 16.5%'; 16.51:33.3='between 16.6% and 33.3% '; else='>33.3%'")
table(CV.eblup1_t)


# dataframe with results for sampled areas
RESULTS<-data.frame(Domain=HT$Domain, n=HT$SampSize, 
                    Dir=HT$Direct, SDdir=HT$SD, CVdir=HT$CV,
                    Eblup=unit.eblup1$est$eblup$eblup, RMSEeblup=sqrt(unit.eblup1$mse$mse),
                    CVeblup=CV.eblup1)

# merge results and shape file
AUT_sae<- merge(Austria, RESULTS, by.x="PB", by.y="Domain")

# spplot(AUT, "Direct", main = "Direct Estimate NUTS 3")
# number of calsses
nclr <- 4
# class <- classIntervals(AUT_sae$Eblup, nclr, style="quantile")
range(RESULTS$Dir)
range(RESULTS$Eblup)
quantile(RESULTS$Dir)
class<- classIntervals(RESULTS$Eblup, n=nclr, style="fixed",
                       fixedBreaks=c(min(RESULTS$Dir), 14498.420,
                                     17041.916, 21580.221,
                                     max(RESULTS$Eblup)))
#breakpoints
categ = class[["brks"]]

plotclr <- brewer.pal(nclr,"YlOrRd")

# plotclr2 <- c("gray72","gray40", "gray17", "gray7")

AUT_sae$cutBHF <- cut(AUT_sae$Eblup, breaks = categ, 
                      right = F, include.lowest=T) 
AUT_sae$cutDir <- cut(AUT_sae$Dir, breaks = categ, 
                      right = F, include.lowest=T) 

spplot(AUT_sae, "cutBHF", col.regions=plotclr, 
       main = "Eblup Estimates (NUTS 3)",
       colorkey = list(space="bottom",height = 0.9, 
                       labels = list(at = seq(0.5, length(categ) -0.5), 
                                     labels = round(categ,2))))

spplot(AUT_sae, "cutDir", col.regions=plotclr, 
       main = "Direct Estimates (NUTS 3)",
       colorkey = list(space="bottom",height = 0.9, 
                       labels = list(at = seq(0.5, length(categ) -0.5), 
                                     labels = round(categ,2))))

#plot all sae estimates 
tmp<-RESULTS[,c(1,6)]
head(tmp)

RESULTS_ALL<- rbind(RESULTS_out,tmp)
  
# merge results and shape file
AUT_sae2<- merge(Austria, RESULTS_ALL, by.x="PB", by.y="Domain")

AUT_sae2$cutBHF <- cut(AUT_sae2$Eblup, breaks = categ, 
                       right = F, include.lowest=T) 
spplot(AUT_sae2, "cutBHF", col.regions=plotclr, 
       main = "Eblup Estimates (NUTS 3)",
       colorkey = list(space="bottom",height = 0.9, 
                       labels = list(at = seq(0.5, length(categ) -0.5), 
                                     labels = round(categ,2))))


# compare estimates
cor(RESULTS$Dir, RESULTS$Eblup)

#bias diagnostic plot
plot(RESULTS$Dir,RESULTS$Eblup, col="black",lwd=1,lty=1,pch=1, 
     main="Bias diagnostic plot", 
     xlab="EBLUP estimate", ylab="Direct survey estimate", ylim = c(9000, 45000))
abline(lm(RESULTS$Dir~RESULTS$Eblup), col="blue",lwd=1,lty=1) 
abline(0,1, col="red",lwd=1,lty=2) 
legend("topleft",legend= c("Fitted line", "45 degree line"),
       lty=c(1,2),col=c("blue","red"),lwd=c(1,2), cex=0.75)
# if the estimates follow the 45 line there are no differences in the estimates. 
# Differently from this plot you can see over-under estimates


#CVs of direct estimates and Eblup estimates
RESULTS<- RESULTS[order(RESULTS$n, na.last = FALSE, decreasing = FALSE), ]
plot(RESULTS$n, RESULTS$CVdir, type = "p",col = "red", 
     lwd = 1, pch = 16, lty = 1, ylab = "CV", 
     xlab = "sample size", cex.axis = 1,
     cex.lab = 1, ylim=c(1,30))
points(RESULTS$n,RESULTS$CVeblup, type = "p", 
       col = "green", lwd = 1, pch = 25, lty = 1)


# CI
#rank districts
RESULTS <- RESULTS[order(RESULTS$Eblup), ] #
RESULTS <- cbind(RESULTS, c(1:dim(RESULTS)[1]))
colnames(RESULTS)[9] <- "rankEBLUP"

library(plotrix)
plotCI(RESULTS$rankEBLUP, RESULTS$Eblup, 
       ui=RESULTS$Eblup+ 1.96*RESULTS$RMSEeblup, 
       li=RESULTS$Eblup- 1.96*RESULTS$RMSEeblup,
       xlab= "Districts (sorted by mean of income)",
       ylab = "Mean of income", 
       main="Confidence Interval Plot", ylim=c(5000,60000),xaxt="n")

plotCI(RESULTS$rankEBLUP+0.5, RESULTS$Dir, 
       ui=RESULTS$Dir+ 1.96*RESULTS$SDdir, 
       li=RESULTS$Dir- 1.96*RESULTS$SDdir, col="red" ,add=T)


text(1:70,RESULTS$Eblup + 2.1*RESULTS$RMSEeblup,RESULTS$Domain, 
     cex=0.6,srt = 90, adj=c(-0.1,0.3))


# FH model
# 

# FH model the mean of eqIncome
X.m1 <- subset(pop, Group.1 %in% sample$district) 

FH.data<-data.frame(Domain=HT$Domain, y.dir=HT$Direct, SD=HT$SD,X.m1[,-1])

area.eblup1<-mseFH(formula=FH.data$y.dir ~ FH.data$gender+ FH.data$eqsize+ 
                   FH.data$cash+ FH.data$self_empl+ FH.data$unempl_ben+FH.data$age_ben, 
                   vardir=FH.data$SD^2)
CV.areaeblup1<-100*sqrt(area.eblup1$mse)/abs(area.eblup1$est$eblup)
summary(CV.areaeblup1)
cor(FH.data$y.dir, area.eblup1$est$eblup)

RESULTS_FH<-data.frame(Domain=HT$Domain, n=HT$SampSize, 
                       Dir=HT$Direct, SDdir=HT$SD, CVdir=HT$CV,
                       Eblup=area.eblup1$est$eblup, 
                       RMSEeblup=sqrt(area.eblup1$mse),
                       CVeblup=CV.areaeblup1)


#bias diagnostic plot
plot(RESULTS_FH$Dir,RESULTS_FH$Eblup, col="black",lwd=1,lty=1,pch=1, 
     main="Bias diagnostic plot", 
     xlab="FH estimate", ylab="Direct survey estimate", ylim = c(9000, 45000))
abline(lm(RESULTS_FH$Dir~RESULTS_FH$Eblup), col="blue",lwd=1,lty=1) 
abline(0,1, col="red",lwd=1,lty=2) 
legend("topleft",legend= c("Fitted line", "45 degree line"),
       lty=c(1,2),col=c("blue","red"),lwd=c(1,2), cex=0.75)


#CVs of direct estimates and Eblup estimates
RESULTS_FH<- RESULTS_FH[order(RESULTS_FH$n, na.last = FALSE, decreasing = FALSE), ]
plot(RESULTS_FH$n, RESULTS_FH$CVdir, type = "p",col = "red", 
     lwd = 1, pch = 16, lty = 1, ylab = "CV", 
     xlab = "sample size", cex.axis = 1,
     cex.lab = 1, ylim=c(1,30))
points(RESULTS_FH$n,RESULTS_FH$CVeblup, type = "p", 
       col = "green", lwd = 1, pch = 25, lty = 1)

# an interesting paper on the comparison of SAE methods is:
# A Comparison of Small Area Estimation Methods for Poverty Mapping
# Author(s): Mara Guadarrama, Isabel Molina, J. N. K. Rao
# http://cejsh.icm.edu.pl/cejsh/element/bwmeta1.element.desklight-4cdb9c32-492c-47af-8eb3-f11dd47a307a